/**
 * apeman task to run browserify
 * @memberof module:apeman-task-browserify/lib
 * @function define
 * @param {object} [options] - Optional settings.
 * @returns {function} - Defined task function.
 */

"use strict";

const argx = require('argx'),
    execcli = require('execcli'),
    expandglob = require('expandglob'),
    objnest = require('objnest');

/** @lends define */
function define(src, options) {
    let args = argx(arguments);
    options = objnest.expand(args.pop('object') || {});
    src = args.remain();

    /**
     * Defined task.
     * @function task
     * @param {object} context - Apeman task context.
     * @param {function} callback - Callback when done.
     */
    function task(context, callback) {
        let logger = context.logger,
            verbose = context.verbose;
        let filenames = src
            .map(src => define.glob(src))
            .reduce((a, b) => [].concat(a, b), []);
        logger.trace(filenames);
        define.browserify([].concat(filenames).concat(options), {}, callback);
    }

    // Description of this task.
    task.$desc = "Execute browserify command.";
    return task;
}

define.browserify = function (args, options, callback) {
    execcli.npmBin('browserify', args, Object.assign({
        notfound: 'try `npm install browserify -g`',
        search: [process.cwd(), __dirname]
    }, options), callback);
};

define.glob = function (src) {
    if (/\*/.test(src)) {
        return expandglob.sync(src);
    }
    return src;
};

module.exports = define;
